@@ -19,9 +19,9 @@
 
 """The main module for the Orca screen reader."""
 
-__id__        = "$Id: orca.py 4109 2008-08-18 16:24:49Z wwalker $"
-__version__   = "$Revision: 4109 $"
-__date__      = "$Date: 2008-08-18 12:24:49 -0400 (Mon, 18 Aug 2008) $"
+__id__        = "$Id$"
+__version__   = "$Revision$"
+__date__      = "$Date$"
 __copyright__ = "Copyright (c) 2004-2008 Sun Microsystems Inc."
 __license__   = "LGPL"
 
@@ -34,6 +34,11 @@ __license__   = "LGPL"
 import sys
 sys.argv[0] = "orca"
 
+# Tell Orca to find/use the right version of pygtk.
+#
+import pygtk
+pygtk.require('2.0')
+
 try:
     # This can fail due to gtk not being available.  We want to
     # be able to recover from that if possible.  The main driver
@@ -1089,12 +1094,9 @@ def helpForOrca(script=None, inputEvent=None):
 
     Returns True to indicate the input event has been consumed.
     """
-
-    props = { gnome.PARAM_APP_DATADIR : '/usr/share'}
-    prog = gnome.program_init('orca', '1.0', properties=props)
-    gnome.help_display_with_doc_id(prog, 'gnome-access-guide',
-                                         'gnome-access-guide', 'ats-2')
-
+    gtk.show_uri(gtk.gdk.screen_get_default(),
+                 "ghelp:gnome-access-guide#ats-2",
+                 gtk.get_current_event_time())
     return True
 
 def quitOrca(script=None, inputEvent=None):
@@ -1223,7 +1225,7 @@ def start(registry):
 
     registry.start()
 
-def abort(exitCode=1):
+def die(exitCode=1):
 
     # We know what we are doing here, so tell pylint not to flag
     # the _exit method call as a warning.  The disable-msg is
@@ -1237,7 +1239,7 @@ def timeout(signum=None, frame=None):
     debug.println(debug.LEVEL_SEVERE,
                   "TIMEOUT: something has hung.  Aborting.")
     debug.printStack(debug.LEVEL_ALL)
-    abort(50)
+    die(50)
 
 def shutdown(script=None, inputEvent=None):
     """Exits Orca.  Unregisters any event listeners and cleans up.  Also
@@ -1305,7 +1307,7 @@ def shutdownOnSignal(signum, frame):
     # something bad is happening, so just quit.
     #
     if exitCount:
-        abort(signum)
+        die(signum)
     else:
         exitCount += 1
 
@@ -1332,13 +1334,13 @@ def shutdownOnSignal(signum, frame):
         signal.alarm(0)
 
     if not cleanExit:
-        abort(signum)
+        die(signum)
 
 def abortOnSignal(signum, frame):
     debug.println(debug.LEVEL_ALL,
                   "Aborting due to signal = %d" \
                   % signum)
-    abort(signum)
+    die(signum)
 
 def usage():
     """Prints out usage information."""
@@ -1541,7 +1543,7 @@ def main():
                     _commandLineSettings["showMainWindow"] = True
                 else:
                     usage()
-                    abort(2)
+                    die(2)
 
             if opt in ("-d", "--disable"):
                 feature = val.strip()
@@ -1557,7 +1559,7 @@ def main():
                     _commandLineSettings["showMainWindow"] = False
                 else:
                     usage()
-                    abort(2)
+                    die(2)
 
             if opt in ("-s", "--gui-setup", "--setup"):
                 setupRequested = True
@@ -1569,21 +1571,21 @@ def main():
                 bypassSetup = True
             if opt in ("-?", "--help"):
                 usage()
-                abort(0)
+                die(0)
             if opt in ("-v", "--version"):
                 print "Orca %s" % platform.version
-                abort(0)
+                die(0)
             if opt in ("-l", "--list-apps"):
                 apps = filter(lambda x: x is not None,
                               pyatspi.Registry.getDesktop(0))
                 for app in apps:
                     print app.name
-                abort(0)
+                die(0)
 
     except:
         debug.printException(debug.LEVEL_OFF)
         usage()
-        abort(2)
+        die(2)
 
     # Do not run Orca if accessibility has not been enabled.
     # We do allow, however, one to force Orca to run via the
@@ -1595,7 +1597,7 @@ def main():
     a11yEnabled = settings.isAccessibilityEnabled()
     if (not bypassSetup) and (not a11yEnabled):
         _showPreferencesConsole()
-        abort()
+        die()
 
     if setupRequested and (not bypassSetup) and (not showGUI):
         _showPreferencesConsole()
