@@ -23,13 +23,12 @@ implementation in Gecko:
 http://developer.mozilla.org/en/docs/Accessibility/ATSPI_Support
 """
 
-__id__        = "$Id: speech_generator.py 4399 2009-01-02 21:02:45Z joanied $"
-__version__   = "$Revision: 4399 $"
-__date__      = "$Date: 2009-01-02 16:02:45 -0500 (Fri, 02 Jan 2009) $"
+__id__        = "$Id$"
+__version__   = "$Revision$"
+__date__      = "$Date$"
 __copyright__ = "Copyright (c) 2005-2008 Sun Microsystems Inc."
 __license__   = "LGPL"
 
-import math
 import pyatspi
 
 import orca.rolenames as rolenames
@@ -38,7 +37,7 @@ import orca.speechgenerator as speechgenerator
 
 from orca.orca_i18n import _
 from orca.orca_i18n import ngettext # for ngettext support
-from orca.orca_i18n import Q_       # to provide qualified translatable strings
+from orca.orca_i18n import C_       # to provide qualified translatable strings
 
 ########################################################################
 #                                                                      #
@@ -80,17 +79,45 @@ class SpeechGenerator(speechgenerator.SpeechGenerator):
 
         utterances = []
         if role or not obj.getRole() in doNotSpeak:
-            utterances.append(rolenames.getSpeechForRoleName(obj, role))
             if obj.getRole() == pyatspi.ROLE_HEADING:
                 level = self._script.getHeadingLevel(obj)
                 if level:
-                    # Translators: this is in reference to a heading level
-                    # in HTML (e.g., For <h3>, the level is 3).
+                    # Translators: the %(level)d is in reference to a heading
+                    # level in HTML (e.g., For <h3>, the level is 3)
+                    # and the %(role)s is in reference to a previously
+                    # translated rolename for the heading.
                     #
-                    utterances.append(_("level %d") % level)
+                    utterances.append(_("%(role)s level %(level)d") % {
+                        'role': rolenames.getSpeechForRoleName(obj, role),
+                        'level': level})
+                else:
+                    utterances.append(rolenames.getSpeechForRoleName(obj, role))
+            else:
+                utterances.append(rolenames.getSpeechForRoleName(obj, role))
 
         return utterances
 
+    def _getSpeechForAlert(self, obj, already_focused):
+        """Gets the speech for an alert.  What we do here is first try
+        to see if the alert contains text via embedded object characters.
+        If it does, we speak that text.  If it doesn't, we defer to the
+        super class.  The prototype alert we're shooting for is the one
+        from http://bugzilla.gnome.org/show_bug.cgi?id=570551
+
+        Arguments:
+        - obj: an Accessible
+        - already_focused: False if object just received focus
+
+        Returns a list of utterances to be spoken for the object.
+        """
+
+        text = self._script.expandEOCs(obj)
+        if text:
+            return [text]
+        else:
+            return speechgenerator.SpeechGenerator.\
+                       _getSpeechForAlert(self, obj, already_focused)
+
     def _getSpeechForDocumentFrame(self, obj, already_focused):
         """Gets the speech for a document frame.
 
@@ -542,16 +569,12 @@ class SpeechGenerator(speechgenerator.SpeechGenerator):
             # Translators: this is in reference to a radio button being
             # selected or not.
             #
-            # ONLY TRANSLATE THE PART AFTER THE PIPE CHARACTER |
-            #
-            selectionState = Q_("radiobutton|selected")
+            selectionState = C_("radiobutton", "selected")
         else:
             # Translators: this is in reference to a radio button being
             # selected or not.
             #
-            # ONLY TRANSLATE THE PART AFTER THE PIPE CHARACTER |
-            #
-            selectionState = Q_("radiobutton|not selected")
+            selectionState = C_("radiobutton", "not selected")
 
         if not already_focused:
             # The label is handled as a context in default.py -- assuming we
@@ -764,40 +787,7 @@ class SpeechGenerator(speechgenerator.SpeechGenerator):
             return speechgenerator.SpeechGenerator.\
                        _getSpeechForSlider(self, obj, already_focused)
 
-        value = obj.queryValue()
-
-        # OK, this craziness is all about trying to figure out the most
-        # meaningful formatting string for the floating point values.
-        # The number of places to the right of the decimal point should
-        # be set by the minimumIncrement, but the minimumIncrement isn't
-        # always set.  So...we'll default the minimumIncrement to 1/100
-        # of the range.  But, if max == min, then we'll just go for showing
-        # them off to two meaningful digits.
-        #
-        try:
-            minimumIncrement = value.minimumIncrement
-        except:
-            minimumIncrement = 0.0
-
-        if minimumIncrement == 0.0:
-            minimumIncrement = (value.maximumValue - value.minimumValue) \
-                               / 100.0
-
-        try:
-            decimalPlaces = max(0, -math.log10(minimumIncrement))
-        except:
-            try:
-                decimalPlaces = max(0, -math.log10(value.minimumValue))
-            except:
-                try:
-                    decimalPlaces = max(0, -math.log10(value.maximumValue))
-                except:
-                    decimalPlaces = 0
-
-        formatter = "%%.%df" % decimalPlaces
-        valueString = formatter % value.currentValue
-        #minString   = formatter % value.minimumValue
-        #maxString   = formatter % value.maximumValue
+        valueString = self._script.getTextForValue(obj)
 
         if already_focused:
             utterances = [valueString]
